@@ -21,8 +21,6 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Based on:                                                               *
- * Version:     $Revision: 1840491 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -966,6 +964,7 @@ static int find_bysession_route(jk_ws_service_t *s,
              * balancer. Of course you will need a some kind of
              * session replication between those two remote.
              */
+            s->sticky = JK_FALSE;
             if (p->sticky_session_force)
                 candidate = -1;
             else if (*wr.redirect) {
