@@ -20,7 +20,6 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 1658173 $                                          *
  ***************************************************************************/
 
 
@@ -111,6 +110,7 @@
 #define DEFAULT_WORKER_TYPE         JK_AJP13_WORKER_NAME
 #define SECRET_KEY_OF_WORKER        "secretkey"
 #define RETRIES_OF_WORKER           "retries"
+#define LB_RETRIES_OF_WORKER        "lb_retries"
 #define STATUS_FAIL_OF_WORKER       "fail_on_status"
 
 #define DEFAULT_WORKER              JK_AJP13_WORKER_NAME
@@ -232,6 +232,7 @@ static const char *unique_properties[] = {
     STYLE_SHEET_OF_WORKER,
     READ_ONLY_OF_WORKER,
     RETRIES_OF_WORKER,
+    LB_RETRIES_OF_WORKER,
     WORKER_MAINTAIN_PROPERTY_NAME,
     NAMESPACE_OF_WORKER,
     XML_NAMESPACE_OF_WORKER,
@@ -342,6 +343,7 @@ static const char *supported_properties[] = {
     BAD_RATING_OF_WORKER,
     SECRET_KEY_OF_WORKER,
     RETRIES_OF_WORKER,
+    LB_RETRIES_OF_WORKER,
     STATUS_FAIL_OF_WORKER,
     LIST_PROPERTY_NAME,
     MAINTAIN_PROPERTY_NAME,
@@ -394,8 +396,6 @@ void jk_sleep(int ms)
     DosSleep(ms);
 #elif defined(BEOS)
     snooze(ms * 1000);
-#elif defined(NETWARE)
-    delay(ms);
 #elif defined(WIN32)
     Sleep(ms);
 #else
@@ -711,13 +711,7 @@ int jk_log(jk_logger_t *l,
     }
 
     if ((l->level <= level) || (level == JK_LOG_REQUEST_LEVEL)) {
-#ifdef NETWARE
-        /* On NetWare, this can get called on a thread that has a limited stack so */
-        /* we will allocate and free the temporary buffer in this function         */
-        char *buf;
-#else
         char buf[LOG_BUFFER_SIZE];
-#endif
         char *f = (char *)(file + strlen(file) - 1);
         va_list args;
         int used = 0;
@@ -729,11 +723,6 @@ int jk_log(jk_logger_t *l,
             f++;
         }
 
-#ifdef NETWARE
-        buf = (char *)malloc(LOG_BUFFER_SIZE);
-        if (NULL == buf)
-            return -1;
-#endif
         used = set_time_str(buf, usable_size, l);
 
         if (line) { /* line==0 only used for request log item */
@@ -747,7 +736,7 @@ int jk_log(jk_logger_t *l,
             used += rc;
             if (rc < 0 ) {
                 strcpy(buf, "Logging failed in pid/tid formatting");
-                l->log(l, level, strlen(buf), buf);
+                l->log(l, level, (int)strlen(buf), buf);
                 return 0;
             }
 
@@ -758,7 +747,7 @@ int jk_log(jk_logger_t *l,
             }
             else {
                 strcpy(buf, "Logging failed in log level formatting");
-                l->log(l, level, strlen(buf), buf);
+                l->log(l, level, (int)strlen(buf), buf);
                 return 0;           /* [V] not sure what to return... */
             }
 
@@ -772,7 +761,7 @@ int jk_log(jk_logger_t *l,
                 }
                 else {
                     strcpy(buf, "Logging failed in function name formatting");
-                    l->log(l, level, strlen(buf), buf);
+                    l->log(l, level, (int)strlen(buf), buf);
                     return 0;           /* [V] not sure what to return... */
                 }
             }
@@ -784,7 +773,7 @@ int jk_log(jk_logger_t *l,
             }
             else {
                 strcpy(buf, "Logging failed in source file name formatting");
-                l->log(l, level, strlen(buf), buf);
+                l->log(l, level, (int)strlen(buf), buf);
                 return 0;           /* [V] not sure what to return... */
             }
 
@@ -793,7 +782,7 @@ int jk_log(jk_logger_t *l,
             used += rc;
             if (rc < 0 || usable_size - used < 0) {
                 strcpy(buf, "Logging failed in line number formatting");
-                l->log(l, level, strlen(buf), buf);
+                l->log(l, level, (int)strlen(buf), buf);
                 return 0;           /* [V] not sure what to return... */
             }
         }
@@ -814,9 +803,6 @@ int jk_log(jk_logger_t *l,
         }
         l->log(l, level, used, buf);
 
-#ifdef NETWARE
-        free(buf);
-#endif
     }
 
     return rc;
@@ -1245,6 +1231,24 @@ int jk_get_worker_retries(jk_map_t *m, const char *wname, int def)
     return rv;
 }
 
+int jk_get_worker_lb_retries(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+    int rv;
+    if (!m || !wname) {
+        return -1;
+    }
+
+    MAKE_WORKER_PARAM(LB_RETRIES_OF_WORKER);
+
+    rv = jk_map_get_int(m, buf, def);
+    if (rv < 1)
+        rv = 1;
+
+    return rv;
+}
+
+
 int jk_get_worker_recovery_opts(jk_map_t *m, const char *wname, int def)
 {
     char buf[PARAM_BUFFER_SIZE];
@@ -1839,7 +1843,7 @@ static int jk_is_some_property(const char *prp_name, const char *suffix, const c
         size_t suffix_len = strlen(suffix);
         size_t sep_len = strlen(sep);
         size_t sep_off = sep_len + suffix_len;
-        
+
         if (prp_name_len >= sep_off) {
             if (!strncmp(prp_name + prp_name_len - sep_off, sep, sep_len) &&
                 !strncmp(prp_name + prp_name_len - suffix_len, suffix, suffix_len)) {
@@ -1910,7 +1914,7 @@ int jk_check_buffer_size()
             max_len = len;
         props++;
     }
-    return JK_MAX_ATTRIBUTE_NAME_LEN - max_len;
+    return JK_MAX_ATTRIBUTE_NAME_LEN - (int)max_len;
 }
 
 /*
@@ -2172,23 +2176,147 @@ int jk_wildchar_match(const char *str, const char *exp, int icase)
     return (str[x] != '\0');
 }
 
-void jk_no2slash(char *name)
+int jk_servlet_normalize(char *path, jk_logger_t *logger)
 {
-    char *d, *s;
+    int l, w;
 
-    s = d = name;
-
-    while (*s) {
-        if ((*d++ = *s) == '/') {
-            do {
-                ++s;
-            } while (*s == '/');
-        }
-        else {
-            ++s;
-        }
+    if (JK_IS_DEBUG_LEVEL(logger)) {
+        jk_log(logger, JK_LOG_DEBUG, "URI on entering jk_servlet_normalize: [%s]", path);
     }
-    *d = '\0';
+
+    // This test allows the loops below to start at index 1 rather than 0.
+    if (path[0] != '/') {
+        if (path[0] == '*' && path[1] == '\0') {
+            /* Most likely an "OPTIONS *" request */
+            return 0;
+        }
+        jk_log(logger, JK_LOG_WARNING, "Uri [%s] does not start with '/'.", path);
+        return JK_NORMALIZE_BAD_PATH;
+    }
+
+    /* First pass.
+     * Remove path parameters ;foo=bar/ from any path segment
+     */
+    for (l = 1, w = 1; path[l] != '\0';) {
+        if (path[l] == ';') {
+            l++;
+            while (path[l] != '/' && path[l] != '\0') {
+                l++;
+            }
+        }
+        else
+            path[w++] = path[l++];
+    }
+    path[w] = '\0';
+
+    /*
+     * Second pass.
+     * Collapse ///// sequences to /
+     */
+    for (l = 1, w = 1; path[l] != '\0';) {
+        if (path[w - 1] == '/' && (path[l] == '/')) {
+            l++;
+        }
+        else
+            path[w++] = path[l++];
+    }
+    path[w] = '\0';
+
+    /* Third pass.
+     * Remove /./ segments
+     * Both leading and trailing segments will be removed.
+     */
+    for (l = 1, w = 1; path[l] != '\0';) {
+        if (path[l] == '.' &&
+                (path[l + 1] == '/' || path[l + 1] == '\0') &&
+                (l == 0 || path[l - 1] == '/')) {
+            l++;
+            if (path[l] == '/') {
+                l++;
+            }
+        }
+        else
+            path[w++] = path[l++];
+    }
+    path[w] = '\0';
+
+    /* Fourth pass.
+     * Remove /xx/../ segments
+     * Trailing segments will be removed but leading /../ segments are an error
+     * condition.
+     */
+    for (l = 1, w = 1; path[l] != '\0';) {
+        if (path[l] == '.' && path[l + 1] == '.' &&
+                (path[l + 2] == '/' || path[l + 2] == '\0') &&
+                (l == 0 || path[l - 1] == '/')) {
+
+            // Wind w back to remove the previous segment
+            if (w == 1) {
+                jk_log(logger,
+                        JK_LOG_EMERG,
+                        "[%s] contains a '/../' sequence that tries to escape above the root.",
+                        path);
+                return JK_NORMALIZE_TRAVERSAL;
+            }
+            do {
+                w--;
+            } while (w != 0 && path[w - 1] != '/');
+
+            // Move l forward to the next segment
+            l += 2;
+
+            if (path[l] == '/') {
+                l++;
+            }
+        }
+        else
+            path[w++] = path[l++];
+    }
+    path[w] = '\0';
+
+    if (JK_IS_DEBUG_LEVEL(logger)) {
+        jk_log(logger, JK_LOG_DEBUG, "URI on exiting jk_servlet_normalize: [%s]", path);
+    }
+
+    return 0;
+}
+
+int jk_strip_session_id(char* path, char* session_name, jk_logger_t *logger) {
+
+    char *jsessionid;
+
+    jsessionid = strstr(path, session_name);
+    if (jsessionid) {
+        int i;
+        int j;
+        if (JK_IS_DEBUG_LEVEL(logger)) {
+            jk_log(logger, JK_LOG_DEBUG,
+                    "removing session identifier for non servlet uri [%s]", path);
+        }
+        // Found a session path parameter.
+        // Need to skip at least as many characters as there are in
+        // strip_session_name
+        i = (int) strlen(session_name);
+        j = 0;
+        // Increment i until the first character after the parameter
+        while (jsessionid[i] != '\0' && jsessionid[i] != ';' && jsessionid[i] != '/') {
+            i++;
+        }
+        // Copy until the end
+        while (jsessionid[i] != '\0') {
+            jsessionid[j++] = jsessionid[i++];
+        }
+        // Terminate
+        jsessionid[j] = '\0';
+
+        if (JK_IS_DEBUG_LEVEL(logger)) {
+            jk_log(logger, JK_LOG_DEBUG,
+                    "result of removing session identifier for non servlet uri is [%s]", path);
+        }
+        return 1;
+    }
+
+    return 0;
 }
 
 #ifdef _MT_CODE_PTHREAD
